<!DOCTYPE html>
<html lang="en">
    <head>
        <meta charset="UTF-8" />
        <meta name="viewport" content="width=device-width, initial-scale=1.0" />
        <meta name="description" content="" />
        <meta name="keywords" content="" />

        <title>(Contact) | MyServer</title>
        <link rel="stylesheet" href="/css/style.css" />
        <script src="/js/app.js"></script>
    </head>

    <body>
        <h1>Welcome to Contact</h1>
        <nav>
            <a href="/">Home</a>
            <a href="/about">About</a>
            <a href="/contact">Contact</a>
        </nav>

        <img src="/images/nancy-2.jpg" height="" width="" alt="Nancy Jewel Mc.Donie" />
    </body>
</html>
